package com.gic.haoban.manage.web.controller;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.view.RedirectView;

import com.alibaba.dubbo.common.URL;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.clerk.api.dto.ClerkDTO;
import com.gic.clerk.api.dto.PowerClerkDTO;
import com.gic.clerk.api.service.ClerkService;
import com.gic.clerk.api.service.UserService;
import com.gic.commons.util.ToolUtil;
import com.gic.enterprise.api.dto.EnterpriseDTO;
import com.gic.enterprise.api.service.EnterpriseService;
import com.gic.haoban.base.api.common.pojo.dto.WebLoginDTO;
import com.gic.haoban.common.anno.IgnoreLogin;
import com.gic.haoban.common.utils.AuthWebRequestUtil;
import com.gic.haoban.common.utils.EntityUtil;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.manage.api.dto.EnterpriseDetailDTO;
import com.gic.haoban.manage.api.dto.StaffDTO;
import com.gic.haoban.manage.api.dto.WxApplicationDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.api.service.StaffApiService;
import com.gic.haoban.manage.api.service.WxApplicationApiService;
import com.gic.haoban.manage.api.service.WxEnterpriseApiService;
import com.gic.haoban.manage.api.service.WxEnterpriseRelatedApiService;
import com.gic.haoban.manage.web.config.Config;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;
import com.gic.haoban.manage.web.vo.StaffVO;
import com.gic.haoban.manage.web.vo.WebLoginVO;
import com.gic.redis.data.util.RedisUtil;
import com.gic.wechat.api.dto.qywx.UserDTO;
import com.gic.wechat.api.service.qywx.QywxCorpApiService;

import cn.hutool.crypto.SecureUtil;
import cn.hutool.crypto.symmetric.AES;

@RestController
public class LoginController extends WebBaseController {

    private static final Logger logger = LoggerFactory.getLogger(LoginController.class);

    private static final String LOGIN_KEY = "haoban#login$key";

    @Autowired
    private StaffApiService staffApiService;
    @Autowired
    private QywxCorpApiService qywxCorpApiService;
    @Autowired
    private WxEnterpriseApiService wxEnterpriseApiService;
    @Autowired
    private WxEnterpriseRelatedApiService wxEnterpriseRelatedApiService;
    @Autowired
    private WxApplicationApiService wxApplicationApiService;
    @Autowired
    private UserService userService;
    @Autowired
    private Config config;
    @Autowired
    private ClerkService clerkService;
    @Autowired
    private EnterpriseService enterpriseService ;

    /**
     * 企微后台登录
     * @param auth_code
     * @param response
     */
    @RequestMapping("login")
    public void login(String auth_code, HttpServletResponse response) {
        String userJson = qywxCorpApiService.getLoginUserInfo(config.getCorpid(), auth_code);
        if (StringUtils.isBlank(userJson)) {
            logger.info("未查询到用户：auth_code:{}", auth_code);
            return;
        }
        UserDTO user = JSON.parseObject(userJson, UserDTO.class);
        String corpId = user.getCorpid();
        logger.info("【企业微信登录】userId={},auth_code={},corpId={}", user.getUserid(), auth_code, corpId);
        WxEnterpriseDTO wxEnterprise = wxEnterpriseApiService.getEnterpriseBycorpId(corpId);
        logger.info("【企业微信】enterprise={}", JSON.toJSONString(wxEnterprise));
        StaffDTO loginStaff = null;
        if (wxEnterprise != null) {
            loginStaff = staffApiService.selectByUserIdAndEnterpriseId(user.getUserid(),
                    wxEnterprise.getWxEnterpriseId());
        }
        if (loginStaff == null) {
            if (wxEnterprise != null) {
                staffApiService.wxFristAdd(JSON.toJSONString(user), wxEnterprise.getWxEnterpriseId());
                loginStaff = staffApiService.selectByUserIdAndEnterpriseId(user.getUserid(),
                        wxEnterprise.getWxEnterpriseId());
            }
        }
        logger.info("【好办web登录】staff={}", JSON.toJSONString(loginStaff));
        if (loginStaff == null) {
            return;
        }
        String wxEnterpriseId = wxEnterprise.getWxEnterpriseId();
        List<EnterpriseDetailDTO> list = this.wxEnterpriseRelatedApiService
                .listEnterpriseByWxEnterpriseId(wxEnterpriseId);
        if (CollectionUtils.isEmpty(list)) {
            logger.info("企微下无关联GIC商户");
            return;
        }
        String gicEnterpriseId = list.get(0).getEnterpriseId();
        String loginPhoneNumber = loginStaff.getPhoneNumber();
        if (StringUtils.isBlank(loginPhoneNumber)) {
            logger.info("未配置手机号,staffId ={}", loginStaff.getStaffId());
            return;
        }
        this.doLogin(wxEnterpriseId, gicEnterpriseId, loginPhoneNumber, "gic", response);
    }

    /**
     * 运维登录
     *
     * @param wxEnterpriseId
     * @param random
     * @param response
     * @return
     */
    @IgnoreLogin
    @RequestMapping("yw-login")
    public HaobanResponse ywLogin(String enterpriseId, String wxEnterpriseId, String random,
            HttpServletResponse response) {
        // 判断是否存在random
        Object o = RedisUtil.getCache(random);
        if (o == null) {
            return resultResponse(HaoBanErrCode.ERR_4);
        }
        if (StringUtils.isBlank(enterpriseId)) {
            List<EnterpriseDetailDTO> list = this.wxEnterpriseRelatedApiService
                    .listEnterpriseByWxEnterpriseId(wxEnterpriseId);
            if (CollectionUtils.isEmpty(list)) {
                return resultResponse(HaoBanErrCode.ERR_4);
            }
            enterpriseId = list.get(0).getEnterpriseId();
        }
        String gicSuperPhone = clerkService.getSuperAdminPhoneNumber(enterpriseId);
        logger.info("登录商户id={},超管手机={}", enterpriseId, gicSuperPhone);
        String result = this.doLogin(wxEnterpriseId, enterpriseId, gicSuperPhone, "yw", response);
        if (StringUtils.isNotBlank(result)) {
            return this.fail(result);
        }
        return null;
    }

    /**
     * gic登录
     *
     * @param random
     * @param wxEnterpriseId
     * @param response
     * @return
     */
    @IgnoreLogin
    @RequestMapping("gic-login")
    public ModelAndView gicLogin(String random, String wxEnterpriseId, HttpServletResponse response) {
        // 判断是否存在random
        AES aes = SecureUtil.aes(LOGIN_KEY.getBytes());
        String o = aes.decryptStr(random);
        if (o == null || StringUtils.isBlank(wxEnterpriseId)) {
            logger.info("解密异常");
            ModelAndView model = new ModelAndView(new RedirectView("/haoban-3/#/gic-error", false, false));
            return model;
        }
        JSONObject jsonObject = JSONObject.parseObject(o);
        String gicEnterpriseId = jsonObject.getString("enterpriseId");
        String loginPhoneNumber = jsonObject.getString("phoneNumber");
        List<WxEnterpriseDTO> enterpriseDTOList = wxEnterpriseRelatedApiService.listWxEnterpriseByEid(gicEnterpriseId,loginPhoneNumber);
        if (CollectionUtils.isEmpty(enterpriseDTOList)) {
            logger.info("登录失败，不存在对应企业:{}", o);
            ModelAndView model = new ModelAndView(new RedirectView("/haoban-3/#/gic-error", false, false));
            return model;
        }

        WxEnterpriseDTO detailDTO = enterpriseDTOList.stream()
                .filter(dto -> dto.getWxEnterpriseId().equals(wxEnterpriseId)).findFirst().orElse(null);

        if (null == detailDTO) {
            logger.info("登录失败，不存在对应企业:gicEnterpriseId:{}", gicEnterpriseId);
            ModelAndView model = new ModelAndView(new RedirectView("/haoban-3/#/gic-error", false, false));
            return model;
        }

        String result = this.doLogin(wxEnterpriseId, gicEnterpriseId, loginPhoneNumber, "gic", response);
        if (StringUtils.isNotBlank(result)) {
            ModelAndView model = new ModelAndView(new RedirectView("/haoban-3/#/gic-error?error="+result, false, false));
            return model;
        }
        return null;
    }

    private String doLogin(String wxEnterpriseId, String gicEnterpriseId, String loginPhoneNumber, String loginChannel,
            HttpServletResponse response) {
        StaffDTO staff = staffApiService.selectByNationcodeAndPhoneNumber(wxEnterpriseId, "86", loginPhoneNumber);
        if (null == staff) {
            logger.info("登录用户手机号不存在!gicEnterpriseId:{}", gicEnterpriseId);
            return "登录失败,好办用户手机号不存在(" + loginPhoneNumber + ")";
        }
        WxApplicationDTO application = wxApplicationApiService.selectByWxEnterpriseIdAndApplicationType(wxEnterpriseId,
                2);
        if (null == application) {
            logger.info("登录企业应用 没授权!gicEnterpriseId:{}", gicEnterpriseId);
            return "登录失败,企业应用没授权";
        }
        WxEnterpriseDTO enterprise = wxEnterpriseApiService.getOne(wxEnterpriseId);
        if (null == enterprise) {
            logger.info("登录失败，不存在对应企业:wxEnterpriseId{}", wxEnterpriseId);
            return "登录失败,不存在对应企业";
        }
        WebLoginDTO webLoginDTO = EntityUtil.changeEntityByJSON(WebLoginDTO.class, staff);
        webLoginDTO.setWxEnterpriseId(wxEnterpriseId);
        webLoginDTO.setEnterpriseId(gicEnterpriseId);
        // 导购id
        ClerkDTO clerk = clerkService.getUserByPhoneNumber(loginPhoneNumber, gicEnterpriseId);
        if (clerk == null) {
            logger.info("gic账号不存在,eid={},phone={}", gicEnterpriseId, loginPhoneNumber);
            return "登录失败,gic账号不存在"+loginPhoneNumber;
        }
        EnterpriseDTO en = this.enterpriseService.getEnterpriseById(gicEnterpriseId) ;
        webLoginDTO.setLoginChannel(loginChannel);
        webLoginDTO.setClerkId(clerk.getClerkId());
        webLoginDTO.setClerkName(clerk.getClerkName());
        webLoginDTO.setBrandName(en.getBrandName());
        webLoginDTO.setSuperAdmin(clerk.getSuperAdmin());
        webLoginDTO.setDepartAuth(clerk.getDepartAuth());
        String redirectUri = config.getHost() + "index";
        redirectUri = redirectUri.replace("haoban-manage3-web", "haoban-3/#");
        String token = ToolUtil.randomUUID() + "-" + ToolUtil.randomStringByTime();
        AuthWebRequestUtil.setSessionUser(webLoginDTO);
        AuthWebRequestUtil.setAppLoginUser(token, webLoginDTO);
        try {
            Cookie cookie = new Cookie("token", token);
            cookie.setPath("/");
            response.addCookie(cookie);
            response.sendRedirect(redirectUri);
            response.setHeader("token", token);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 获取登录信息
     * @return res
     */
    @RequestMapping("get-login-info")
    public HaobanResponse getLoginInfo() {
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        logger.info("当前登录信息={}",JSON.toJSON(loginUser));
        WebLoginVO  vo = EntityUtil.changeEntityByOrika(WebLoginVO.class, loginUser) ;
        String staffId = loginUser.getStaffId() ;
        StaffDTO dto = this.staffApiService.selectById(staffId) ;
        vo.setStaffName(dto.getStaffName()+"("+loginUser.getClerkName()+")");
        StaffVO staffVO = EntityUtil.changeEntityByOrika(StaffVO.class, dto) ;
        vo.setStaffVO(staffVO);
        return this.success(vo) ;
    }
    
    @RequestMapping("get-login-session")
    @IgnoreLogin
    public HaobanResponse getLoginSession() {
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        return this.success(loginUser) ;
    }

    /**
     * 切换GIC商户
     * @param enterpriseId
     * @return
     */
    @RequestMapping("enterprise-is-over")
    public HaobanResponse enterpriseIsOver(String enterpriseId) {
        if (StringUtils.isEmpty(enterpriseId)) {
            return this.fail("企业ID空") ;
        }
        EnterpriseDTO enterprise = this.enterpriseService.getEnterpriseById(enterpriseId);
        if(null == enterprise) {
            return this.fail("gic商户查询不到") ;
        }
        String token = AuthWebRequestUtil.getToken();
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        if(null == loginUser) {
            return this.fail("用户未登录") ;
        }
        logger.info("切换商户,登录信息={},enterpriseId",JSON.toJSONString(loginUser),enterpriseId);
        String phoneNumber = loginUser.getPhoneNumber() ;
        ClerkDTO clerk = clerkService.getUserByPhoneNumber(phoneNumber, enterpriseId);
        if (clerk == null) {
            logger.info("切换商户失败,gic账号不存在,eid={},phone={}", enterpriseId, phoneNumber);
            return this.fail("切换商户失败,gic账号不存在") ;
        }
        loginUser.setEnterpriseId(enterpriseId);
        loginUser.setClerkId(clerk.getClerkId());
        loginUser.setClerkName(clerk.getClerkName());
        loginUser.setSuperAdmin(clerk.getSuperAdmin());
        loginUser.setDepartAuth(clerk.getDepartAuth());
        loginUser.setBrandName(enterprise.getBrandName());
        AuthWebRequestUtil.setSessionUser(loginUser);
        AuthWebRequestUtil.setAppLoginUser(token, loginUser);
        boolean flag = wxEnterpriseApiService.enterpriseIsOver(enterpriseId);
        if(flag) {
            logger.info("切换商户失败,商户已过期,eid={},phone={}", enterpriseId, phoneNumber);
            return this.fail("切换商户失败,商户已过期") ;
        }
        return resultResponse(HaoBanErrCode.ERR_1, false);
    }

    /**
     * 好办-扫码登录
     * @return
     * @throws UnsupportedEncodingException
     */
    @RequestMapping("get-login-qrcode")
    public HaobanResponse getLoginQrcode() throws UnsupportedEncodingException {
        String corpid = config.getCorpid();
        String redirectUri = config.getHost() + "login";
        String url = "https://open.work.weixin.qq.com/wwopen/sso/3rd_qrConnect?appid=" + corpid + "&redirect_uri="
                + URLEncoder.encode(redirectUri, "UTF-8") + "&state=web_login@gyoss9&usertype=admin";
        return resultResponse(HaoBanErrCode.ERR_1, URL.decode(url));
    }

    /**
     * 登录微信企业列表
     * @param random
     * @return
     */
    @RequestMapping("login-wx-enterprise-list")
    public HaobanResponse loginWxEnterpriseList(String random) {
        // 判断是否存在random
        AES aes = SecureUtil.aes(LOGIN_KEY.getBytes());
        String o = aes.decryptStr(random);
        if (o == null) {
            logger.info("解密异常");
            return resultResponse(HaoBanErrCode.ERR_5);
        }
        logger.info(o);
        JSONObject jsonObject = JSONObject.parseObject(o);
        String gicEnterpriseId = jsonObject.getString("enterpriseId");
        String phoneNumber = jsonObject.getString("phoneNumber") ;
        String wxEnterpriseId = jsonObject.getString("wxEnterpriseId");
        List<WxEnterpriseDTO> enterpriseDTOList = wxEnterpriseRelatedApiService.listWxEnterpriseByEid(gicEnterpriseId,phoneNumber);
        if (CollectionUtils.isEmpty(enterpriseDTOList)) {
            logger.info("登录失败，不存在对应企业:{}", o);
            return resultResponse(HaoBanErrCode.ERR_6);
        }
        if (StringUtils.isNotBlank(wxEnterpriseId)) {
            List<WxEnterpriseDTO> collect = enterpriseDTOList.stream()
                    .filter(dto -> dto.getWxEnterpriseId().equals(wxEnterpriseId)).collect(Collectors.toList());
            return resultResponse(HaoBanErrCode.ERR_1, collect);
        }
        return resultResponse(HaoBanErrCode.ERR_1, enterpriseDTOList);
    }

    /**
     * 登出
     * @return
     */
    @RequestMapping("login-out")
    public HaobanResponse loginOut() {
        String token = "";
        Cookie[] cookies = AuthWebRequestUtil.getRequest().getCookies();
        if (cookies != null) {
            for (Cookie cookie : cookies) {
                if (cookie.getName().equals("token")) {
                    token = cookie.getValue();
                    break;
                }
            }
        }
        if (StringUtils.isNotBlank(token)) {
            RedisUtil.delCache(token);
        }
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    @RequestMapping("/gic-login-check")
    public ModelAndView gicLogin(String code, String hbEnt) {
        logger.info("gic登录clerkId={},hbEnt={}", code, hbEnt);
        PowerClerkDTO clerkDTO = this.userService.haobanLoginBack(code);
        if (clerkDTO == null) {
            logger.info("登录失败,传入clerkId={}无效", code);
            ModelAndView model = new ModelAndView(new RedirectView("/haoban-web/#/gic-error?errCode=1", false));
            return model;
        }
        String gicEnterpriseId = clerkDTO.getEnterpriseId();
        String loginPhoneNumber = clerkDTO.getPhoneNumber();
        Map<String, String> retMap = new HashMap<>();
        retMap.put("enterpriseId", gicEnterpriseId);
        retMap.put("phoneNumber", loginPhoneNumber);
        retMap.put("wxEnterpriseId", hbEnt);
        String ret = JSONObject.toJSONString(retMap);
        AES aes = SecureUtil.aes(LOGIN_KEY.getBytes());
        String random = aes.encryptHex(ret);
        logger.info("使用新好办3：{}", random);
        ModelAndView model = new ModelAndView(
                new RedirectView("/haoban-3/#/loginMore?random=" + random, false, false));
        return model;
    }

}
